# flake8: noqa

# import apis into api package
from activitysmith_openapi.api.live_activities_api import LiveActivitiesApi
from activitysmith_openapi.api.metrics_api import MetricsApi
from activitysmith_openapi.api.push_notifications_api import PushNotificationsApi




# coding: utf-8

"""
    ActivitySmith API

    Send push notifications and Live Activities to your own devices via a single API key.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from activitysmith_openapi.models.metric_format import MetricFormat
from activitysmith_openapi.models.metric_unit_spacing import MetricUnitSpacing
from activitysmith_openapi.models.widget_metric_latest_value import WidgetMetricLatestValue
from typing import Optional, Set
from typing_extensions import Self

class WidgetMetric(BaseModel):
    """
    A metric configured for ActivitySmith widgets.
    """ # noqa: E501
    public_id: StrictStr
    key: Annotated[str, Field(min_length=1, strict=True, max_length=64)]
    label: Annotated[str, Field(min_length=1, strict=True, max_length=80)]
    currency_code: Optional[Annotated[str, Field(min_length=3, strict=True, max_length=3)]] = Field(description="Present when format is currency.")
    unit: Optional[Annotated[str, Field(strict=True, max_length=16)]] = Field(description="Present when format is unit.")
    unit_spacing: MetricUnitSpacing
    format: MetricFormat
    latest_value: Optional[WidgetMetricLatestValue]
    latest_value_at: Optional[datetime]
    created_at: datetime
    updated_at: datetime
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["public_id", "key", "label", "currency_code", "unit", "unit_spacing", "format", "latest_value", "latest_value_at", "created_at", "updated_at"]

    @field_validator('key')
    def key_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-z0-9][a-z0-9_.-]{0,63}$", value):
            raise ValueError(r"must validate the regular expression /^[a-z0-9][a-z0-9_.-]{0,63}$/")
        return value

    @field_validator('currency_code')
    def currency_code_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[A-Z]{3}$", value):
            raise ValueError(r"must validate the regular expression /^[A-Z]{3}$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of WidgetMetric from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of latest_value
        if self.latest_value:
            _dict['latest_value'] = self.latest_value.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if currency_code (nullable) is None
        # and model_fields_set contains the field
        if self.currency_code is None and "currency_code" in self.model_fields_set:
            _dict['currency_code'] = None

        # set to None if unit (nullable) is None
        # and model_fields_set contains the field
        if self.unit is None and "unit" in self.model_fields_set:
            _dict['unit'] = None

        # set to None if latest_value (nullable) is None
        # and model_fields_set contains the field
        if self.latest_value is None and "latest_value" in self.model_fields_set:
            _dict['latest_value'] = None

        # set to None if latest_value_at (nullable) is None
        # and model_fields_set contains the field
        if self.latest_value_at is None and "latest_value_at" in self.model_fields_set:
            _dict['latest_value_at'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of WidgetMetric from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "public_id": obj.get("public_id"),
            "key": obj.get("key"),
            "label": obj.get("label"),
            "currency_code": obj.get("currency_code"),
            "unit": obj.get("unit"),
            "unit_spacing": obj.get("unit_spacing"),
            "format": obj.get("format"),
            "latest_value": WidgetMetricLatestValue.from_dict(obj["latest_value"]) if obj.get("latest_value") is not None else None,
            "latest_value_at": obj.get("latest_value_at"),
            "created_at": obj.get("created_at"),
            "updated_at": obj.get("updated_at")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    ActivitySmith API

    Send push notifications and Live Activities to your own devices via a single API key.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from activitysmith_openapi.models.metric_value_update_request import MetricValueUpdateRequest

class TestMetricValueUpdateRequest(unittest.TestCase):
    """MetricValueUpdateRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> MetricValueUpdateRequest:
        """Test MetricValueUpdateRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `MetricValueUpdateRequest`
        """
        model = MetricValueUpdateRequest()
        if include_optional:
            return MetricValueUpdateRequest(
                value = None,
                timestamp = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f')
            )
        else:
            return MetricValueUpdateRequest(
                value = None,
        )
        """

    def testMetricValueUpdateRequest(self):
        """Test MetricValueUpdateRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    ActivitySmith API

    Send push notifications and Live Activities to your own devices via a single API key.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from activitysmith_openapi.models.widget_metric import WidgetMetric

class TestWidgetMetric(unittest.TestCase):
    """WidgetMetric unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> WidgetMetric:
        """Test WidgetMetric
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `WidgetMetric`
        """
        model = WidgetMetric()
        if include_optional:
            return WidgetMetric(
                public_id = '',
                key = 'w6otgfww_r74cm.fyln9uyrj1gzi.i4v8m.oy3c3lv1-42d1z89qs7dlt2ggwxdm0',
                label = '0',
                currency_code = 'AEI012',
                unit = '',
                unit_spacing = 'none',
                format = 'number',
                latest_value = 1.337,
                latest_value_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f')
            )
        else:
            return WidgetMetric(
                public_id = '',
                key = 'w6otgfww_r74cm.fyln9uyrj1gzi.i4v8m.oy3c3lv1-42d1z89qs7dlt2ggwxdm0',
                label = '0',
                currency_code = 'AEI012',
                unit = '',
                unit_spacing = 'none',
                format = 'number',
                latest_value = 1.337,
                latest_value_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
        )
        """

    def testWidgetMetric(self):
        """Test WidgetMetric"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    ActivitySmith API

    Send push notifications and Live Activities to your own devices via a single API key.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from activitysmith_openapi.models.metric_unit_spacing import MetricUnitSpacing

class TestMetricUnitSpacing(unittest.TestCase):
    """MetricUnitSpacing unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testMetricUnitSpacing(self):
        """Test MetricUnitSpacing"""
        # inst = MetricUnitSpacing()

if __name__ == '__main__':
    unittest.main()



from activitysmith.client import ActivitySmith
import activitysmith.client as client_module


class FakePushNotificationsApi:
    def __init__(self, _api_client):
        self._api_client = _api_client
        self.calls = []

    def send_push_notification(self, **kwargs):
        self.calls.append(kwargs)
        return kwargs


class FakeLiveActivitiesApi:
    def __init__(self, _api_client):
        self._api_client = _api_client
        self.calls = []

    def start_live_activity(self, **kwargs):
        self.calls.append(("start", kwargs))
        return kwargs

    def update_live_activity(self, **kwargs):
        self.calls.append(("update", kwargs))
        return kwargs

    def end_live_activity(self, **kwargs):
        self.calls.append(("end", kwargs))
        return kwargs

    def reconcile_live_activity_stream(self, **kwargs):
        self.calls.append(("stream", kwargs))
        return kwargs

    def end_live_activity_stream(self, **kwargs):
        self.calls.append(("end_stream", kwargs))
        return kwargs


class FakeMetricsApi:
    def __init__(self, _api_client):
        self._api_client = _api_client
        self.calls = []

    def update_metric_value(self, **kwargs):
        self.calls.append(kwargs)
        return kwargs


def test_sdk_header_and_user_agent_are_configured(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")
    api_client = client.metrics._api._api_client

    assert api_client.default_headers["X-ActivitySmith-SDK"] == "python-v1.2.0"
    assert api_client.default_headers["User-Agent"] == "activitysmith-python/1.2.0"


def test_notifications_short_and_legacy_alias(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")
    payload = {"title": "Build Failed"}

    short = client.notifications.send(payload)
    legacy = client.notifications.send_push_notification(payload)

    assert short == {"push_notification_request": payload}
    assert legacy == {"push_notification_request": payload}
    assert client.notifications._api.calls == [
        {"push_notification_request": payload},
        {"push_notification_request": payload},
    ]


def test_notifications_map_channels_to_target(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")
    payload = {"title": "Build Failed", "channels": ["devs", "ops"]}

    client.notifications.send(payload)
    client.notifications.send_push_notification({"title": "Build Failed", "channels": "devs,ops"})

    assert client.notifications._api.calls == [
        {"push_notification_request": {"title": "Build Failed", "target": {"channels": ["devs", "ops"]}}},
        {"push_notification_request": {"title": "Build Failed", "target": {"channels": ["devs", "ops"]}}},
    ]


def test_notifications_preserve_media_and_redirection(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")
    payload = {
        "title": "Voice Over Generated",
        "media": "https://cdn.activitysmith.com/voice_over.mp3",
        "redirection": "https://studio.acme.com/voice-overs/482/review",
    }

    client.notifications.send(payload)

    assert client.notifications._api.calls == [
        {"push_notification_request": payload},
    ]


def test_notifications_reject_media_and_actions(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")

    try:
        client.notifications.send(
            {
                "title": "Voice Over Generated",
                "media": "https://cdn.activitysmith.com/voice_over.mp3",
                "actions": [{"title": "Open", "type": "open_url", "url": "https://example.com"}],
            }
        )
    except ValueError as exc:
        assert str(exc) == "ActivitySmith: media cannot be combined with actions"
    else:
        raise AssertionError("Expected ValueError for media + actions")


def test_live_activities_short_and_legacy_aliases(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")
    start_payload = {
        "content_state": {
            "title": "Deploy",
            "number_of_steps": 4,
            "current_step": 1,
            "type": "segmented_progress",
        }
    }
    update_payload = {
        "activity_id": "act-1",
        "content_state": {"title": "Deploy", "current_step": 2},
    }
    end_payload = {
        "activity_id": "act-1",
        "content_state": {"title": "Deploy", "current_step": 4},
    }

    client.live_activities.start(start_payload)
    client.live_activities.update(update_payload)
    client.live_activities.end(end_payload)

    client.live_activities.start_live_activity(start_payload)
    client.live_activities.update_live_activity(update_payload)
    client.live_activities.end_live_activity(end_payload)

    assert client.live_activities._api.calls == [
        ("start", {"live_activity_start_request": start_payload}),
        ("update", {"live_activity_update_request": update_payload}),
        ("end", {"live_activity_end_request": end_payload}),
        ("start", {"live_activity_start_request": start_payload}),
        ("update", {"live_activity_update_request": update_payload}),
        ("end", {"live_activity_end_request": end_payload}),
    ]


def test_live_activities_start_maps_channels_to_target(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")
    payload = {
        "content_state": {
            "title": "Deploy",
            "number_of_steps": 4,
            "current_step": 1,
            "type": "segmented_progress",
        },
        "channels": ["devs", "ops"],
    }

    client.live_activities.start(payload)
    client.live_activities.start_live_activity(payload)

    expected = {
        "content_state": {
            "title": "Deploy",
            "number_of_steps": 4,
            "current_step": 1,
            "type": "segmented_progress",
        },
        "target": {"channels": ["devs", "ops"]},
    }
    assert client.live_activities._api.calls == [
        ("start", {"live_activity_start_request": expected}),
        ("start", {"live_activity_start_request": expected}),
    ]


def test_live_activities_support_progress_payloads(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")
    payload = {
        "content_state": {
            "title": "Render export",
            "subtitle": "encoding frames",
            "type": "progress",
            "percentage": 67,
            "color": "purple",
        }
    }

    client.live_activities.start(payload)

    assert client.live_activities._api.calls == [
        ("start", {"live_activity_start_request": payload}),
    ]


def test_live_activities_stream_short_and_legacy_aliases(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")
    stream_payload = {
        "content_state": {
            "title": "Server Health",
            "subtitle": "prod-web-1",
            "type": "metrics",
            "metrics": [
                {"label": "CPU", "value": 9, "unit": "%"},
                {"label": "MEM", "value": 45, "unit": "%"},
            ],
        },
        "channels": ["ops"],
    }
    end_payload = {
        "content_state": {
            "title": "Server Health",
            "subtitle": "prod-web-1",
            "type": "metrics",
            "metrics": [
                {"label": "CPU", "value": 7, "unit": "%"},
                {"label": "MEM", "value": 38, "unit": "%"},
            ],
        }
    }

    client.live_activities.stream("prod-web-1", stream_payload)
    client.live_activities.end_stream("prod-web-1", end_payload)
    client.live_activities.reconcile_live_activity_stream("prod-web-1", stream_payload)
    client.live_activities.end_live_activity_stream("prod-web-1", end_payload)

    expected_stream = {
        "content_state": stream_payload["content_state"],
        "target": {"channels": ["ops"]},
    }
    assert client.live_activities._api.calls == [
        (
            "stream",
            {
                "stream_key": "prod-web-1",
                "live_activity_stream_request": expected_stream,
            },
        ),
        (
            "end_stream",
            {
                "stream_key": "prod-web-1",
                "live_activity_stream_delete_request": end_payload,
            },
        ),
        (
            "stream",
            {
                "stream_key": "prod-web-1",
                "live_activity_stream_request": expected_stream,
            },
        ),
        (
            "end_stream",
            {
                "stream_key": "prod-web-1",
                "live_activity_stream_delete_request": end_payload,
            },
        ),
    ]


def test_live_activities_pass_action_payloads_through(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")

    start_payload = {
        "content_state": {
            "title": "Deploying payments-api",
            "subtitle": "Running database migrations",
            "number_of_steps": 5,
            "current_step": 3,
            "type": "segmented_progress",
        },
        "action": {
            "title": "Open Workflow",
            "type": "open_url",
            "url": "https://github.com/acme/payments-api/actions/runs/1234567890",
        },
    }

    update_payload = {
        "activity_id": "act-1",
        "content_state": {
            "title": "Reindexing product search",
            "subtitle": "Shard 7 of 12",
            "number_of_steps": 12,
            "current_step": 7,
        },
        "action": {
            "title": "Pause Reindex",
            "type": "webhook",
            "url": "https://ops.example.com/hooks/search/reindex/pause",
            "method": "POST",
            "body": {"job_id": "reindex-2026-03-19"},
        },
    }

    end_payload = {
        "activity_id": "act-1",
        "content_state": {
            "title": "Deploying payments-api",
            "subtitle": "Production rollout complete",
            "number_of_steps": 5,
            "current_step": 5,
        },
        "action": {
            "title": "Open Workflow",
            "type": "open_url",
            "url": "https://github.com/acme/payments-api/actions/runs/1234567890",
        },
    }

    client.live_activities.start(start_payload)
    client.live_activities.update(update_payload)
    client.live_activities.end(end_payload)

    assert client.live_activities._api.calls == [
        ("start", {"live_activity_start_request": start_payload}),
        ("update", {"live_activity_update_request": update_payload}),
        ("end", {"live_activity_end_request": end_payload}),
    ]


def test_metrics_short_and_legacy_aliases(monkeypatch):
    monkeypatch.setattr(client_module, "PushNotificationsApi", FakePushNotificationsApi)
    monkeypatch.setattr(client_module, "LiveActivitiesApi", FakeLiveActivitiesApi)
    monkeypatch.setattr(client_module, "MetricsApi", FakeMetricsApi)

    client = ActivitySmith(api_key="x")

    client.metrics.update(
        "deploy.success_rate",
        99.9,
        timestamp="2026-05-03T12:30:00.000Z",
    )
    client.metrics.update("prod.status", {"value": "healthy"})
    client.metrics.update_metric_value("deploy.success_rate", {"value": 42})

    assert client.metrics._api.calls == [
        {
            "key": "deploy.success_rate",
            "metric_value_update_request": {
                "value": 99.9,
                "timestamp": "2026-05-03T12:30:00.000Z",
            },
        },
        {
            "key": "prod.status",
            "metric_value_update_request": {"value": "healthy"},
        },
        {
            "key": "deploy.success_rate",
            "metric_value_update_request": {"value": 42},
        },
    ]
